@@ -9,7 +9,8 @@ LIB_USER=/usr/lib/aif/user
 RUNTIME_DIR=/tmp/aif
 LOG_DIR=/var/log/aif
 LOGFILE=$LOG_DIR/aif.log
-
+DISCLAIMER="Note that despite our careful coding and proper testing there may still be bugs in this software.\n
+When you are doing this installation on a system where some data must be preserved, we suggest you make a backup first"
 
 ###### Early bootstrap ######
 
@@ -33,7 +34,7 @@ process_args ()
 echo "Welcome to $TITLE"
 
 mount -o remount,rw / &>/dev/null 
-
+cleanup_runtime
 
 ### Set configuration values ###
 # note : you're free to use or ignore these in your procedure.  probably you want to use these variables to override defaults in your configure worker
@@ -128,6 +129,7 @@ done
 
 # Set pacman vars.  allow procedures to have set $var_TARGET_DIR (TODO: look up how delayed variable substitution works. then we can put this at the top again)
 # flags like --noconfirm should not be specified here.  it's up to the procedure to decide the interactivity
+# NOTE: Pacman will run with currently active locale, if you want to parse output, you should prefix with LANG=C
 PACMAN=pacman
 PACMAN_TARGET="pacman --root $var_TARGET_DIR --config /tmp/pacman.conf"
 
