@@ -3,7 +3,7 @@
 
 # TODO: implement 'retry until user does it correctly' everywhere
 # TODO: at some places we should check if $1 etc is only 1 word because we often depend on that
-# TODO: standardize. eg everything $1= question/title, $2=default
+# TODO: standardize. eg everything $1= question/title, $2=default. also default should probably be 'no' for no default everywhere
 # TODO: figure out something to make dia windows always big enough, yet fit nicely in the terminal
 
 
@@ -169,17 +169,15 @@ ask_timezone ()
 		REGIONS="$REGIONS $i -"
 	done
 	while true; do
-		ask_option no "Please select a region" '' required $REGIONS
+		ask_option no "Please select a region" '' required $REGIONS || return 1
 		region=$ANSWER_OPTION
-		if [ $? -eq 0 ]; then
-			ZONES=""
-			for i in $(grep '^[A-Z]' /usr/share/zoneinfo/zone.tab | grep $region/ | cut -f 3 | sed -e "s#$region/##g"| sort -u); do
-				ZONES="$ZONES $i -"
-			done
-			ask_option no "Please select a timezone" '' required $ZONES
-			zone=$ANSWER_OPTION
-			[ $? -eq 0 ] && ANSWER_TIMEZONE="$region/$zone" && return
-		fi
+		ZONES=""
+		for i in $(grep '^[A-Z]' /usr/share/zoneinfo/zone.tab | grep $region/ | cut -f 3 | sed -e "s#$region/##g"| sort -u); do
+			ZONES="$ZONES $i -"
+		done
+		ask_option no "Please select a timezone" '' required $ZONES || return 1
+		zone=$ANSWER_OPTION
+		ANSWER_TIMEZONE="$region/$zone" && return
 	done
 }
 
@@ -323,7 +321,7 @@ _dia_ask_checklist ()
 		shift 3
 	done
 	_dia_dialog --checklist "$str" 0 0 0 $list 2>$ANSWER
-	ret=$?
+	local ret=$?
 	ANSWER_CHECKLIST=`cat $ANSWER`
 	debug 'UI' "_dia_ask_checklist: user checked ON: $ANSWER_CHECKLIST"
 	return $ret
@@ -356,18 +354,18 @@ _dia_ask_number ()
 		[ -n $3 -a $3 != '0' ] && str2="$str2 max $3"
 		[ -n "$str2" ] && str="$str ( $str2 )"
 		_dia_dialog --inputbox "$str" 0 0 $4 2>$ANSWER
-		ret=$?
+		local ret=$?
 		ANSWER_NUMBER=`cat $ANSWER`
 		if [[ $ANSWER_NUMBER = *[^0-9]* ]] #TODO: handle exit state
 		then
-			show_warning "$ANSWER_NUMBER is not a number! try again."
+			show_warning 'Invalid number input' "$ANSWER_NUMBER is not a number! try again."
 		else
 			if [ -n "$3" -a $3 != '0' -a $ANSWER_NUMBER -gt $3 ]
 			then
-				show_warning "$ANSWER_NUMBER is bigger then the maximum,$3! try again."
+				show_warning 'Invalid number input' "$ANSWER_NUMBER is bigger then the maximum,$3! try again."
 			elif [ -n "$2" -a $ANSWER_NUMBER -lt $2 ]
 			then
-				show_warning "$ANSWER_NUMBER is smaller then the minimum,$2! try again."
+				show_warning 'Invalid number input' "$ANSWER_NUMBER is smaller then the minimum,$2! try again."
 			else
 				break
 			fi
@@ -395,7 +393,7 @@ _dia_ask_option ()
 	CANCEL_LABEL=Cancel
 	[ $TYPE == optional ] && CANCEL_LABEL='Skip'
 	_dia_dialog $DEFAULT --cancel-label $CANCEL_LABEL --colors --title " $DIA_MENU_TITLE " --menu "$DIA_MENU_TEXT $EXTRA_INFO" 0 0 0 "$@" 2>$ANSWER
-	ret=$?
+	local ret=$?
 	ANSWER_OPTION=`cat $ANSWER`
 	debug 'UI' "dia_ask_option: ANSWER_OPTION: $ANSWER_OPTION, returncode (skip/cancel): $ret ($DIA_MENU_TITLE)"
 	[ $TYPE == required ] && return $ret
@@ -415,7 +413,7 @@ _dia_ask_password ()
 	fi
 
 	_dia_dialog --passwordbox  "Enter your $type_l password" 8 65 "$2" 2>$ANSWER
-	ret=$?
+	local ret=$?
 	[ -n "$type_u" ] && read ${type_u}_PASSWORD < $ANSWER
 	[ -z "$type_u" ] && read           PASSWORD < $ANSWER
 	cat $ANSWER
@@ -428,7 +426,7 @@ _dia_ask_string ()
 {
 	exitcode=${3:-1}
 	_dia_dialog --inputbox "$1" 0 0 "$2" 2>$ANSWER
-	ret=$?
+	local ret=$?
 	ANSWER_STRING=`cat $ANSWER`
 	debug 'UI' "_dia_ask_string: user entered $ANSWER_STRING"
 	[ -z "$ANSWER_STRING" ] && return $exitcode
@@ -439,12 +437,12 @@ _dia_ask_string ()
 
 _dia_ask_yesno ()
 {
-	height=$((`echo -e "$1" | wc -l` +7))
+	local default
 	str=$1
-	#TODO: i think dialog doesnt support a default value for yes/no, so we need this workaround:
-	[ -n "$2" ] && str="$str (default: $2)"
-	dialog --yesno "$str" $height 55 # returns 0 for yes, 1 for no
-	ret=$?
+	# If $2 contains an explicit 'no' we set defaultno for yesno dialog
+	[ "$2" == "no" ] && default="--defaultno"
+	dialog $default --yesno "$str" 0 0 # returns 0 for yes, 1 for no
+	local ret=$?
 	[ $ret -eq 0 ] && debug 'UI' "dia_ask_yesno: User picked YES"
 	[ $ret -gt 0 ] && debug 'UI' "dia_ask_yesno: User picked NO"
 	return $ret
@@ -515,14 +513,14 @@ _cli_ask_number ()
 		read ANSWER_NUMBER
 		if [[ $ANSWER_NUMBER = *[^0-9]* ]]
 		then
-			show_warning "$ANSWER_NUMBER is not a number! try again."
+			show_warning 'Invalid number input' "$ANSWER_NUMBER is not a number! try again."
 		else
 			if [ -n "$3" -a $3 != '0' -a $ANSWER_NUMBER -gt $3 ]
 			then
-				show_warning "$ANSWER_NUMBER is bigger then the maximum,$3! try again."
+				show_warning 'Invalid number input' "$ANSWER_NUMBER is bigger then the maximum,$3! try again."
 			elif [ -n "$2" -a $ANSWER_NUMBER -lt $2 ]
 			then
-				show_warning "$ANSWER_NUMBER is smaller then the minimum,$2! try again."
+				show_warning 'Invalid number input' "$ANSWER_NUMBER is smaller then the minimum,$2! try again."
 			else
 				break
 			fi
@@ -563,7 +561,7 @@ _cli_ask_option ()
 	[ -n "$DEFAULT" ] && echo -n " > [ $DEFAULT ] "
 	[ -z "$DEFAULT" ] && echo -n " > "
 	read ANSWER_OPTION
-	ret=0
+	local ret=0
 	[ -z "$ANSWER_OPTION" -a -n "$DEFAULT" ] && ANSWER_OPTION="$DEFAULT"
 	[ "$ANSWER_OPTION" == CANCEL ] && ret=1 && ANSWER_OPTION=
 	[ "$ANSWER_OPTION" == SKIP   ] && ret=0 && ANSWER_OPTION=
